//Module : MAILSLOT.CPP
// Autor: Sasa Popovic
// Version: 1.1
// Date: 24.02.2003
// Email: sasap@uns.ns.ac.yu
// If there are any suggestions or comments,
// you are most welcome to contact me



/////////////////////////////////  Includes  //////////////////////////////////
#include "stdafx.h"
#include "mailslot.h"



#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#define new DEBUG_NEW
#endif



////////////////////////////////// Implementation /////////////////////////////

CServerMailslot::CServerMailslot()
{
  m_hMailslot = INVALID_HANDLE_VALUE;
}

CServerMailslot::~CServerMailslot()
{
  Close();
}

BOOL CServerMailslot::Close()
{
	BOOL bSuccess = TRUE;
  if (IsOpen())
  {
    bSuccess = CloseHandle(m_hMailslot);
    m_hMailslot = INVALID_HANDLE_VALUE;
  }

  if (!bSuccess)
    TRACE1("CServerMailslot::Close() failed, GetLastError returned %d\n", GetLastError());

  return bSuccess;
}

BOOL CServerMailslot::IsOpen() const
{
  return (m_hMailslot != INVALID_HANDLE_VALUE);
}

BOOL CServerMailslot::Open(LPCTSTR lpszName, 
                           DWORD dwMaxMessageSize, 
                           DWORD dwReadTimeout, 
                           LPSECURITY_ATTRIBUTES lpSecurityAttributes)
{
  ASSERT(!IsOpen());

  //the class encapsulates creating the mailslot name, all that is required is
  //a simple name for the mailslot e.g. lpName = PJSLOT will create the mailslot
  //name \\.\mailslot\PJSLOT
  TCHAR pszMailslotName[_MAX_PATH];
  _tcscpy(pszMailslotName, _T("\\\\.\\mailslot\\"));
  _tcscat(pszMailslotName, lpszName);

  m_hMailslot = CreateMailslot(pszMailslotName, dwMaxMessageSize, dwReadTimeout, lpSecurityAttributes);

  if (m_hMailslot == INVALID_HANDLE_VALUE)
    TRACE1("CServerMailslot::Open() failed, GetLastError returned %d\n", GetLastError());

  return (m_hMailslot != INVALID_HANDLE_VALUE);
}

BOOL CServerMailslot::MessageWaiting()
{
  ASSERT(IsOpen());
  DWORD dwCount = 0;
  BOOL bSuccess = GetMailslotInfo(m_hMailslot, NULL, NULL, &dwCount, NULL);

  if (!bSuccess)
    TRACE1("CServerMailslot::MessageWaiting() failed, GetLastError returned %d\n", GetLastError());

  return bSuccess && (dwCount != 0);
}

DWORD CServerMailslot::MessageCount()
{
  ASSERT(IsOpen());
  DWORD dwCount = 0;
  BOOL bSuccess = GetMailslotInfo(m_hMailslot, NULL, NULL, &dwCount, NULL);

  if (!bSuccess)
    TRACE1("CServerMailslot::MessageCount() failed, GetLastError returned %d\n", GetLastError());

  if (bSuccess)
    return dwCount;
  else
    return 0;
}

DWORD CServerMailslot::SizeOfWaitingMessage()
{
  ASSERT(IsOpen());
  DWORD dwSize = 0;
  BOOL bSuccess = GetMailslotInfo(m_hMailslot, NULL, &dwSize, NULL, NULL);

  if (!bSuccess)
    TRACE1("CServerMailslot::SizeOfWaitingMessage() failed, GetLastError returned %d\n", GetLastError());

  if (bSuccess)
    return dwSize;
  else
    return 0;
}

DWORD CServerMailslot::MaximumMessageSize()
{
  ASSERT(IsOpen());
  DWORD dwMaxSize = 0;
  BOOL bSuccess = GetMailslotInfo(m_hMailslot, &dwMaxSize, NULL, NULL, NULL);

  if (!bSuccess)
    TRACE1("CServerMailslot::MaxmimumMessageSize() failed, GetLastError returned %d\n", GetLastError());

  if (bSuccess)
    return dwMaxSize;
  else
    return 0;
}

BOOL CServerMailslot::SetReadTimeout(DWORD dwReadTimeout)
{
  ASSERT(IsOpen());
  BOOL bSuccess = SetMailslotInfo(m_hMailslot, dwReadTimeout);

  if (!bSuccess)
    TRACE1("CServerMailslot::SetReadTimeout() failed, GetLastError returned %d\n", GetLastError());

  return bSuccess;
}

BOOL CServerMailslot::Read(LPVOID lpBuffer, DWORD dwNumberOfBytesToRead, DWORD& dwNumberOfBytesRead)
{
  ASSERT(IsOpen());
  BOOL bSuccess = ReadFile(m_hMailslot, lpBuffer, dwNumberOfBytesToRead, &dwNumberOfBytesRead, (LPOVERLAPPED) NULL);

  if (!bSuccess)
    TRACE1("CServerMailslot::Read() failed, GetLastError returned %d\n", GetLastError());

  return bSuccess;
}

BOOL CServerMailslot::GetCreationTime(LPFILETIME lpCreationTime)
{
  ASSERT(IsOpen());
  BOOL bSuccess = GetFileTime(m_hMailslot, lpCreationTime, NULL, NULL);

  if (!bSuccess)
    TRACE1("CServerMailslot::GetCreationTime() failed, GetLastError returned %d\n", GetLastError());

  return bSuccess;
}

BOOL CServerMailslot::GetLastAccessTime(LPFILETIME lpLastAccessTime)
{
  ASSERT(IsOpen());
  BOOL bSuccess = GetFileTime(m_hMailslot, NULL, lpLastAccessTime, NULL);

  if (!bSuccess)
    TRACE1("CServerMailslot::GetLastAccessTime() failed, GetLastError returned %d\n", GetLastError());

  return bSuccess;
}

BOOL CServerMailslot::GetLastWriteTime(LPFILETIME lpLastWriteTime)
{
  ASSERT(IsOpen());
  BOOL bSuccess = GetFileTime(m_hMailslot, NULL, NULL, lpLastWriteTime);

  if (!bSuccess)
    TRACE1("CServerMailslot::GetLastWriteTime() failed, GetLastError returned %d\n", GetLastError());

  return bSuccess;
}

#ifdef _DEBUG
void CServerMailslot::AssertValid() const
{
  CObject::AssertValid();
  ASSERT(IsOpen());
}
#endif

#ifdef _DEBUG
void CServerMailslot::Dump(CDumpContext& dc) const
{
  CObject::Dump(dc);

  CString sText;
  sText.Format(_T("Open=%d, m_hMailslot=%x\n"), IsOpen(), m_hMailslot);
  dc << sText;
}
#endif